@@ -37,7 +37,7 @@ struct Task {
 impl Woke for Task {
     fn wake_by_ref(_: &Arc<Self>) {
         // poll everything because future is done and may have created conditions for something to finish
-        run()
+        DEFAULT_EXECUTOR.lock().poll_tasks()
     }
 }
 
@@ -47,6 +47,17 @@ impl GlobalExecutor for Executor {
         self.add_task(future);
         self.poll_tasks();
     }
+}
+
+impl Executor {
+    /// Add task for a future to the list of tasks
+    fn add_task(&mut self, future: Box<dyn Future<Output = ()> + 'static + Send + Unpin>) {
+        // store our task
+        let task = Arc::new(Task {
+            future: Mutex::new(Box::pin(future)),
+        });
+        self.tasks.push(task);
+    }
 
     // Poll all tasks on global executor
     fn poll_tasks(&mut self) {
@@ -71,22 +82,26 @@ impl GlobalExecutor for Executor {
     }
 }
 
-impl Executor {
-    /// Add task for a future to the list of tasks
-    fn add_task(&mut self, future: Box<dyn Future<Output = ()> + 'static + Send + Unpin>) {
-        // store our task
-        let task = Arc::new(Task {
-            future: Mutex::new(Box::pin(future)),
-        });
-        self.tasks.push(task);
+struct DefaultExecutor;
+
+impl GlobalExecutor for DefaultExecutor {
+    // Add a task on the global executor
+    fn spawn(&mut self, future: Box<dyn Future<Output = ()> + 'static + Send + Unpin>) {
+        DEFAULT_EXECUTOR.lock().spawn(future)
     }
 }
 
+
 lazy_static! {
-    static ref GLOBAL_EXECUTOR: Mutex<Box<dyn GlobalExecutor+Send+Sync>> = {
+    static ref DEFAULT_EXECUTOR: Mutex<Box<Executor>> = {
         let m = Executor::default();
         Mutex::new(Box::new(m))
     };
+
+    static ref GLOBAL_EXECUTOR: Mutex<Box<dyn GlobalExecutor+Send+Sync>> = {
+        let m = DefaultExecutor;
+        Mutex::new(Box::new(m))
+    };
 }
 
 /// Give future to global executor to be polled and executed.
@@ -100,11 +115,6 @@ pub fn set_global_executor(executor:impl GlobalExecutor+Send+Sync+'static) {
     *global_executor = Box::new(executor);
 }
 
-fn run() {
-    GLOBAL_EXECUTOR.lock().poll_tasks()
-}
-
 pub trait GlobalExecutor {
     fn spawn(&mut self, future: Box<dyn Future<Output = ()> + 'static + Send + Unpin>);
-    fn poll_tasks(&mut self);
 }
\ No newline at end of file
